#!/bin/bash

############################################################################################
#
#Created by Serge Yakimchuck - yakim@yakim.org.ua
#Nagios plugin for testing https certificate
#Also can test DNS CAA record
#In case error CAA return WARNING status
#
############################################################################################

PORT=443
DOMAIN=''
WARNING=30
CRITICAL=7
hostflag=false
caaflag=true
ERR=0
NEED_CAA=0
CAA_OUT=''

help_message () {
 echo "Could not parse arguments"
	   echo Usage: check_cert.sh -H hostname [pdwcC]
	   echo -H hostname or IP address mandatory parameter
	   echo -p port default 443
	   echo -d domain name default is a same as HOSTNAME
	   echo -w warning value in days  default 30
	   echo -c critical value in days default 7
	   echo -C test CAA record. Value 1 - check CAA record
	   exit 2

}

test_caa () {
CAA=`nslookup -type=caa $DOMAIN |grep issuewild|awk -F \" '{print $2}'`
if [ -z "$CAA" ] ;
then
CAA=`nslookup -type=caa $DOMAIN |grep issue|grep -v issuewild|awk -F \" '{print $2}'`
fi

if [ -z "$CAA" ] ;
then

PARENT_DOM=`echo $DOMAIN| awk -F '.' '{OFS="."} {$1=""; print $0}'|cut -c 2-`

CAA=`nslookup -type=caa $PARENT_DOM |grep issuewild|awk -F \" '{print $2}'`

if [ -z "$CAA" ] ;
then
caaflag=false
CAA_OUT="No CAA record"
ERR=1
return 1
fi

fi



#test cert
CERT_CA=`echo | openssl s_client -servername $DOMAIN  -connect $HOST:$PORT 2>/dev/null | openssl x509 -text| grep "$CAA"`
if [ -z "$CERT_CA" ] ;
then
CAA_OUT="Wrong CAA record"
ERR=1
return 1
fi

ERR=0
CAA_OUT="CAA record - OK"

}


while getopts :w:H:p:c:d:C: flag
do
    case "${flag}" in
        H) hostflag=true; HOST=${OPTARG};;
        w) WARNING=${OPTARG};;
        c) CRITICAL=${OPTARG};;
	d) DOMAIN=${OPTARG};;
	p) PORT=${OPTARG};;
	C) NEED_CAA=${OPTARG};;
	:) help_message;;
	*) help_message
    esac
done


if ! $hostflag 
then
    echo
    echo argument -H is mandatory
    echo
    help_message
fi


if [ -z "$DOMAIN" ];
then
DOMAIN=$HOST
fi

if [ -z "$CRITICAL" ];
then
CRITICAL=7
fi


if [ -z "$WARNING" ];
then
WARNING=30
fi


if [ "$NEED_CAA" -eq "1" ];
then
 test_caa;
fi


DOMAIN_EXIST=`echo | openssl s_client -servername $DOMAIN  -connect $HOST:$PORT 2>/dev/null | openssl x509 -text| grep "DNS:$DOMAIN"`
if [ -z "$DOMAIN_EXIST" ];
then
OUT=No certificate for $DOMAIN
exit 2
fi


END_DATE=`echo |openssl s_client -servername $DOMAIN -connect $HOST:$PORT 2>/dev/null | openssl x509 -noout -dates|grep -i notafter|awk -F "=" '{print $2}'`
#echo $END_DATE
#echo | openssl s_client -connect yakim.org.ua:443 2>/dev/null | openssl x509 -noout -dates

DATE_DIF=$(( ($(date -d "$END_DATE"  +%s) - $(date +%s)) / 86400 ))

OUT="Certificate for the $DOMAIN will expired in $DATE_DIF days"
if [ "$DATE_DIF" -le "$CRITICAL" ]
then
  ERR=2
#fi

elif [ "$DATE_DIF" -le "$WARNING" ]
then
  ERR=1
fi
echo $CAA_OUT"  "$OUT
exit $ERR

